#include <lua.h>
#include <lauxlib.h>

#include "volksdata.h"

#ifndef _LUA_VOLK_H
#define _LUA_VOLK_H

/// Raise Lua error including VOLK error message on negative rc.
#define LUA_PCHECK(exp, message) do {\
    if (UNLIKELY ((exp) < VOLK_OK)) \
        return luaL_error (L, "%s: %s", message, VOLK_strerror (exp)); \
} while (0)

#define LUA_NLCHECK(exp, ...) do {\
    if (UNLIKELY ((exp) == NULL)) return luaL_error (L, __VA_ARGS__); \
} while (0)


/*
 * Type check macros.
 */

#define check_term(L) \
    *(VOLK_Term **)luaL_checkudata(L, 1, "VOLK.Term")

#define check_triple(L) \
    *(VOLK_Triple **)luaL_checkudata(L, 1, "VOLK.Triple")

#define check_graph(L) \
    *(VOLK_Graph **)luaL_checkudata(L, 1, "VOLK.Graph")


/// Enum (int) constants to be passed to the module.
typedef struct l_enum_const {
    const char  *k;
    const int   v;
} LEnumConst;


/// String constants to be passed to the module.
typedef struct l_string_const {
    const char  *k;
    const char  *v;
} LStringConst;


void push_int_const (lua_State *L, const LEnumConst *list);
void push_string_const (lua_State *L, const LStringConst *list);

/// Create new namespace map.
int l_nsmap_new (lua_State *L);

/// Allocate space for a term object.
VOLK_Term **allocate_term (lua_State *L);

/// Convert a VOLK_TermSet to a table and push it.
VOLK_rc
term_set_to_table (lua_State *L, VOLK_TermSet *ts);

#endif  // _LUA_VOLK_H
